@@ -37,10 +37,16 @@ module Kubeclient
 
     SEARCH_ARGUMENTS = {
       'labelSelector' => :label_selector,
-      'fieldSelector' => :field_selector
+      'fieldSelector' => :field_selector,
+      'limit'         => :limit,
+      'continue'      => :continue
     }.freeze
 
-    WATCH_ARGUMENTS = { 'resourceVersion' => :resource_version }.merge!(SEARCH_ARGUMENTS).freeze
+    WATCH_ARGUMENTS = {
+      'labelSelector'   => :label_selector,
+      'fieldSelector'   => :field_selector,
+      'resourceVersion' => :resource_version
+    }.freeze
 
     attr_reader :api_endpoint
     attr_reader :ssl_options
@@ -126,21 +132,47 @@ module Kubeclient
     end
 
     def self.parse_definition(kind, name)
-      # "name": "componentstatuses", networkpolicies, endpoints
-      # "kind": "ComponentStatus" NetworkPolicy, Endpoints
-      # maintain pre group api compatibility for endpoints and securitycontextconstraints.
-      # See: https://github.com/kubernetes/kubernetes/issues/8115
-      kind = kind[0..-2] if %w[Endpoints SecurityContextConstraints].include?(kind)
+      # Kubernetes gives us 3 inputs:
+      #   kind: "ComponentStatus", "NetworkPolicy", "Endpoints"
+      #   name: "componentstatuses", "networkpolicies", "endpoints"
+      #   singularName: "componentstatus" etc (usually omitted, defaults to kind.downcase)
+      # and want to derive singular and plural method names, with underscores:
+      #   "network_policy"
+      #   "network_policies"
+      # kind's CamelCase word boundaries determine our placement of underscores.
 
-      prefix = kind[0..kind.rindex(/[A-Z]/)] # NetworkP
-      m = name.match(/^#{prefix.downcase}(.*)$/)
-      m && OpenStruct.new(
-        entity_type:   kind, # ComponentStatus
-        resource_name: name, # componentstatuses
-        method_names:  [
-          ClientMixin.underscore_entity(kind),         # component_status
-          ClientMixin.underscore_entity(prefix) + m[1] # component_statuses
-        ]
+      if IRREGULAR_NAMES[kind]
+        # In a few cases, the given kind / singularName itself is still plural.
+        # We require a distinct singular method name, so force it.
+        method_names = IRREGULAR_NAMES[kind]
+      else
+        # TODO: respect singularName from discovery?
+        # But how?  If it differs from kind.downcase, kind's word boundaries don't apply.
+        singular_name = kind.downcase
+
+        if !(/[A-Z]/ =~ kind)
+          # Some custom resources have a fully lowercase kind - can't infer underscores.
+          method_names = [singular_name, name]
+        else
+          # Some plurals are not exact suffixes, e.g. NetworkPolicy -> networkpolicies.
+          # So don't expect full last word to match.
+          /^(?<prefix>(.*[A-Z]))(?<singular_suffix>[^A-Z]*)$/ =~ kind  # "NetworkP", "olicy"
+          if name.start_with?(prefix.downcase)
+            plural_suffix = name[prefix.length..-1]                    # "olicies"
+            prefix_underscores = ClientMixin.underscore_entity(prefix) # "network_p"
+            method_names = [prefix_underscores + singular_suffix,      # "network_policy"
+                            prefix_underscores + plural_suffix]        # "network_policies"
+          else
+            # Something weird, can't infer underscores for plural so just give them up
+            method_names = [singular_name, name]
+          end
+        end
+      end
+
+      OpenStruct.new(
+        entity_type:   kind,
+        resource_name: name,
+        method_names:  method_names
       )
     end
 
@@ -256,6 +288,8 @@ module Kubeclient
     #   :namespace (string) - the namespace of the entity.
     #   :label_selector (string) - a selector to restrict the list of returned objects by labels.
     #   :field_selector (string) - a selector to restrict the list of returned objects by fields.
+    #   :limit (integer) - a maximum number of items to return in each response
+    #   :continue (string) - a token used to retrieve the next chunk of entities
     #   :as (:raw|:ros) - defaults to :ros
     #     :raw - return the raw response body as a string
     #     :ros - return a collection of RecursiveOpenStruct objects
@@ -313,9 +347,7 @@ module Kubeclient
       # TODO: temporary solution to add "kind" and apiVersion to request
       # until this issue is solved
       # https://github.com/GoogleCloudPlatform/kubernetes/issues/6439
-      # TODO: #2 solution for
-      # https://github.com/kubernetes/kubernetes/issues/8115
-      hash[:kind] = (entity_type.eql?('Endpoint') ? 'Endpoints' : entity_type)
+      hash[:kind] = entity_type
       hash[:apiVersion] = @api_group + @api_version
       response = handle_exception do
         rest_client[ns_prefix + resource_name]
@@ -401,12 +433,7 @@ module Kubeclient
           @entities[kind.to_s].resource_name
         end
       ns_prefix = build_namespace_prefix(namespace)
-      # TODO: Change this once services supports the new scheme
-      if entity_name_plural == 'pods'
-        rest_client["#{ns_prefix}#{entity_name_plural}/#{name}:#{port}/proxy"].url
-      else
-        rest_client["proxy/#{ns_prefix}#{entity_name_plural}/#{name}:#{port}"].url
-      end
+      rest_client["#{ns_prefix}#{entity_name_plural}/#{name}:#{port}/proxy"].url
     end
 
     def process_template(template)
@@ -432,7 +459,15 @@ module Kubeclient
 
     private
 
-    # Format ditetime according to RFC3339
+    IRREGULAR_NAMES = {
+      # In a few cases, the given kind itself is still plural.
+      # https://github.com/kubernetes/kubernetes/issues/8115
+      'Endpoints'                  => %w[endpoint endpoints],
+      'SecurityContextConstraints' => %w[security_context_constraint
+                                         security_context_constraints]
+    }.freeze
+
+    # Format datetime according to RFC3339
     def format_datetime(value)
       case value
       when DateTime, Time
@@ -461,10 +496,14 @@ module Kubeclient
               result.fetch('metadata', {}).fetch('resourceVersion', nil)
             end
 
+          # If 'limit' was passed save the continue token
+          # see https://kubernetes.io/docs/reference/using-api/api-concepts/#retrieving-large-results-sets-in-chunks
+          continue = result.fetch('metadata', {}).fetch('continue', nil)
+
           # result['items'] might be nil due to https://github.com/kubernetes/kubernetes/issues/13096
           collection = result['items'].to_a.map { |item| Kubeclient::Resource.new(item) }
 
-          Kubeclient::Common::EntityList.new(list_type, resource_version, collection)
+          Kubeclient::Common::EntityList.new(list_type, resource_version, collection, continue)
         else
           Kubeclient::Resource.new(result)
         end
@@ -477,6 +516,9 @@ module Kubeclient
       @entities = {}
       fetch_entities['resources'].each do |resource|
         next if resource['name'].include?('/')
+        # Not a regular entity, special functionality covered by `process_template`.
+        # https://github.com/openshift/origin/issues/21668
+        next if resource['kind'] == 'Template' && resource['name'] == 'processedtemplates'
         resource['kind'] ||=
           Kubeclient::Common::MissingKindCompatibility.resource_kind(resource['name'])
         entity = ClientMixin.parse_definition(resource['kind'], resource['name'])
