@@ -10,3 +10,8 @@ require 'kubeclient'
 def open_test_file(name)
   File.new(File.join(File.dirname(__FILE__), name.split('.').last, name))
 end
+
+def stub_core_api_list
+  stub_request(:get, %r{/api/v1$})
+    .to_return(body: open_test_file('core_api_resource_list.json'), status: 200)
+end
