@@ -3,7 +3,7 @@ require_relative 'test_helper'
 # Watch entity tests
 class TestWatch < MiniTest::Test
   def test_watch_pod_success
-    stub_resource_list
+    stub_core_api_list
 
     expected = [
       { 'type' => 'ADDED', 'resourceVersion' => '1389' },
@@ -28,7 +28,7 @@ class TestWatch < MiniTest::Test
   end
 
   def test_watch_pod_raw
-    stub_resource_list
+    stub_core_api_list
 
     stub_request(:get, %r{/watch/pods}).to_return(
       body: open_test_file('watch_stream.json'),
@@ -43,7 +43,7 @@ class TestWatch < MiniTest::Test
   end
 
   def test_watch_pod_failure
-    stub_resource_list
+    stub_core_api_list
     stub_request(:get, %r{/watch/pods}).to_return(status: 404)
 
     client = Kubeclient::Client.new('http://localhost:8080/api/', 'v1')
@@ -72,7 +72,7 @@ class TestWatch < MiniTest::Test
   def test_watch_with_resource_version
     api_host = 'http://localhost:8080/api'
     version = '1995'
-    stub_resource_list
+    stub_core_api_list
     stub_request(:get, %r{.*\/watch/events})
       .to_return(body: open_test_file('watch_stream.json'),
                  status: 200)
@@ -90,7 +90,7 @@ class TestWatch < MiniTest::Test
     api_host = 'http://localhost:8080/api'
     selector = 'name=redis-master'
 
-    stub_resource_list
+    stub_core_api_list
     stub_request(:get, %r{.*\/watch/events})
       .to_return(body: open_test_file('watch_stream.json'),
                  status: 200)
@@ -108,7 +108,7 @@ class TestWatch < MiniTest::Test
     api_host = 'http://localhost:8080/api'
     selector = 'involvedObject.kind=Pod'
 
-    stub_resource_list
+    stub_core_api_list
     stub_request(:get, %r{.*\/watch/events})
       .to_return(body: open_test_file('watch_stream.json'),
                  status: 200)
@@ -125,7 +125,7 @@ class TestWatch < MiniTest::Test
   def test_watch_with_finish_and_ebadf
     api_host = 'http://localhost:8080/api'
 
-    stub_resource_list
+    stub_core_api_list
     stub_request(:get, %r{.*\/watch/events})
       .to_return(body: open_test_file('watch_stream.json'), status: 200)
 
@@ -140,13 +140,4 @@ class TestWatch < MiniTest::Test
 
     assert_requested(:get, "#{api_host}/v1/watch/events", times: 1)
   end
-
-  private
-
-  def stub_resource_list
-    stub_request(:get, %r{/api/v1$}).to_return(
-      body: open_test_file('core_api_resource_list.json'),
-      status: 200
-    )
-  end
 end
